from _typeshed import Unused
from _typeshed.wsgi import WSGIApplication
from collections.abc import Callable, Iterable, Mapping
from typing import Any, Literal

from waitress.adjustments import _AdjustmentsParams
from waitress.server import BaseWSGIServer

def serve(
    app: WSGIApplication,
    *,
    _server: Callable[..., BaseWSGIServer] = ...,
    _quiet: bool = False,
    _profile: bool = False,
    **kw: _AdjustmentsParams,
) -> None: ...
def serve_paste(app: WSGIApplication, global_conf: Unused, **kw: _AdjustmentsParams) -> Literal[0]: ...
def profile(cmd: str, globals: dict[str, Any], locals: Mapping[str, Any], sort_order: Iterable[str], callers: bool) -> None: ...



from _typeshed import ReadableBuffer
from io import BufferedIOBase, BufferedRandom, BytesIO
from typing import Final, Literal, NoReturn

COPY_BYTES: Final = 262144
STRBUF_LIMIT: Final = 8192

class FileBasedBuffer:
    remain: int
    file: BytesIO
    def __init__(self, file: BytesIO, from_buffer: BytesIO | None = None) -> None: ...
    def __len__(self) -> int: ...
    def __bool__(self) -> Literal[True]: ...
    def append(self, s: ReadableBuffer) -> None: ...
    def get(self, numbytes: int = -1, skip: bool = False) -> bytes: ...
    def skip(self, numbytes: int, allow_prune: int = 0) -> None: ...
    def newfile(self) -> BufferedIOBase: ...
    def prune(self) -> None: ...
    def getfile(self) -> BytesIO: ...
    def close(self) -> None: ...

class TempfileBasedBuffer(FileBasedBuffer):
    def __init__(self, from_buffer: BytesIO | None = None) -> None: ...
    def newfile(self) -> BufferedRandom: ...

class BytesIOBasedBuffer(FileBasedBuffer):
    file: BytesIO
    def __init__(self, from_buffer: BytesIO | None = None) -> None: ...
    def newfile(self) -> BytesIO: ...

class ReadOnlyFileBasedBuffer(FileBasedBuffer):
    file: BytesIO
    block_size: int
    def __init__(self, file: BytesIO, block_size: int = 32768) -> None: ...
    remain: int
    def prepare(self, size: int | None = None) -> int: ...
    def get(self, numbytes: int = -1, skip: bool = False) -> bytes: ...
    def __iter__(self) -> ReadOnlyFileBasedBuffer: ...
    def next(self) -> bytes | None: ...
    __next__ = next
    def append(self, s: ReadableBuffer) -> NoReturn: ...

class OverflowableBuffer:
    overflowed: bool
    buf: BufferedIOBase | None
    strbuf: bytes
    overflow: int
    def __init__(self, overflow: int) -> None: ...
    def __len__(self) -> int: ...
    def __bool__(self) -> bool: ...
    def append(self, s: bytes) -> None: ...
    def get(self, numbytes: int = -1, skip: bool = False) -> bytes: ...
    def skip(self, numbytes: int, allow_prune: bool = False) -> None: ...
    def prune(self) -> None: ...
    def getfile(self) -> BytesIO: ...
    def close(self) -> None: ...



import sys
from _typeshed import ReadableBuffer
from collections.abc import Mapping
from io import BytesIO
from logging import Logger
from socket import _RetAddress, socket
from typing_extensions import TypeAlias

_Socket: TypeAlias = socket
_SocketMap: TypeAlias = Mapping[int, socket]

socket_map: _SocketMap

class ExitNow(Exception): ...

def read(obj: dispatcher) -> None: ...
def write(obj: dispatcher) -> None: ...
def readwrite(obj: dispatcher, flags: int) -> None: ...
def poll(timeout: float = 0.0, map: _SocketMap | None = None) -> None: ...
def poll2(timeout: float = 0.0, map: _SocketMap | None = None) -> None: ...

poll3 = poll2

def loop(timeout: float = 30.0, use_poll: bool = False, map: _SocketMap | None = None, count: int | None = None) -> None: ...
def compact_traceback() -> tuple[tuple[str, str, str], BaseException, BaseException, str]: ...

class dispatcher:
    debug: bool
    connected: bool
    accepting: bool
    connecting: bool
    closing: bool
    addr: _RetAddress | None
    ignore_log_types: frozenset[str]
    logger: Logger
    @staticmethod
    def compact_traceback() -> tuple[tuple[str, str, str], BaseException, BaseException, str]: ...
    socket: socket | None
    def __init__(self, sock: _Socket | None = None, map: _SocketMap | None = None) -> None: ...
    def add_channel(self, map: _SocketMap | None = None) -> None: ...
    def del_channel(self, map: _SocketMap | None = None) -> None: ...
    family_and_type: tuple[int, int]
    def create_socket(self, family: int = ..., type: int = ...) -> None: ...
    def set_socket(self, sock: _Socket, map: _SocketMap | None = None) -> None: ...
    def set_reuse_addr(self) -> None: ...
    def readable(self) -> bool: ...
    def writable(self) -> bool: ...
    def listen(self, num: int) -> None: ...
    def bind(self, addr: _RetAddress) -> None: ...
    def accept(self) -> tuple[_Socket, _RetAddress] | None: ...
    def send(self, data: bytes, do_close: bool = True) -> int: ...
    def recv(self, buffer_size: int) -> bytes: ...
    def close(self) -> None: ...
    def log(self, message: str) -> None: ...
    def log_info(self, message: str, type: str = "info") -> None: ...
    def handle_read_event(self) -> None: ...
    def handle_connect_event(self) -> None: ...
    def handle_write_event(self) -> None: ...
    def handle_expt_event(self) -> None: ...
    def handle_error(self) -> None: ...
    def handle_expt(self) -> None: ...
    def handle_read(self) -> None: ...
    def handle_write(self) -> None: ...
    def handle_connect(self) -> None: ...
    def handle_accept(self) -> None: ...
    def handle_accepted(self, sock: _Socket, addr: _RetAddress) -> None: ...
    def handle_close(self) -> None: ...

def close_all(map: _SocketMap | None = None, ignore_all: bool = False) -> None: ...

if sys.platform != "win32":
    class file_wrapper:
        fd: BytesIO
        def __init__(self, fd: BytesIO) -> None: ...
        def __del__(self) -> None: ...
        def recv(self, length: int, /) -> bytes: ...
        def send(self, data: ReadableBuffer, /) -> bytes: ...
        def getsockopt(self, level: int, optname: int, buflen: bool | None = None) -> int: ...
        read = recv
        write = send
        def close(self) -> None: ...
        def fileno(self) -> BytesIO: ...

    class file_dispatcher(dispatcher):
        connected: bool
        def __init__(self, fd: BytesIO, map: _SocketMap | None = None) -> None: ...
        socket: socket
        def set_file(self, fd: BytesIO) -> None: ...



import sys
from _typeshed import Unused
from _typeshed.wsgi import WSGIApplication
from collections.abc import Callable, Sequence
from socket import _RetAddress, socket
from typing import Literal

from waitress import wasyncore
from waitress.adjustments import Adjustments, _AdjustmentsParams
from waitress.channel import HTTPChannel
from waitress.task import Task, ThreadedTaskDispatcher
from waitress.wasyncore import _SocketMap

def create_server(
    application: WSGIApplication,
    map: _SocketMap | None = None,
    _start: bool = True,
    _sock: socket | None = None,
    _dispatcher: ThreadedTaskDispatcher | None = None,
    **kw: _AdjustmentsParams,
) -> MultiSocketServer | BaseWSGIServer: ...

class MultiSocketServer:
    asyncore = wasyncore
    adj: Adjustments
    map: _SocketMap | None
    effective_listen: Sequence[tuple[str, int]]
    task_dispatcher: ThreadedTaskDispatcher
    def __init__(
        self,
        map: _SocketMap | None = None,
        adj: Adjustments | None = None,
        effective_listen: Sequence[tuple[str, int]] | None = None,
        dispatcher: ThreadedTaskDispatcher | None = None,
        # Can be None, but print_listen will fail
        log_info: Callable[[str], Unused] | None = None,
    ) -> None: ...
    def print_listen(self, format_str: str) -> None: ...
    def run(self) -> None: ...
    def close(self) -> None: ...

class BaseWSGIServer(wasyncore.dispatcher):
    channel_class: type[HTTPChannel]
    next_channel_cleanup: int
    socketmod: socket
    asyncore = wasyncore
    in_connection_overflow: bool
    sockinfo: tuple[int, int, int | None, _RetAddress]
    family: int
    socktype: int
    application: WSGIApplication
    adj: Adjustments
    trigger: int
    task_dispatcher: ThreadedTaskDispatcher
    server_name: str
    active_channels: HTTPChannel
    def __init__(
        self,
        application: WSGIApplication,
        map: _SocketMap | None = None,
        _start: bool = True,
        _sock: socket | None = None,
        dispatcher: ThreadedTaskDispatcher | None = None,
        adj: Adjustments | None = None,
        sockinfo: tuple[int, int, int | None, _RetAddress] | None = None,
        bind_socket: bool = True,
        **kw: _AdjustmentsParams,
    ) -> None: ...
    def bind_server_socket(self) -> None: ...
    def getsockname(self) -> tuple[str, str]: ...
    accepting: bool
    def accept_connections(self) -> None: ...
    def add_task(self, task: Task) -> None: ...
    def readable(self) -> bool: ...
    def writable(self) -> bool: ...
    def handle_read(self) -> None: ...
    def handle_connect(self) -> None: ...
    def handle_accept(self) -> None: ...
    def run(self) -> None: ...
    def pull_trigger(self) -> None: ...
    def set_socket_options(self, conn: socket) -> None: ...
    def fix_addr(self, addr: _RetAddress) -> _RetAddress: ...
    def maintenance(self, now: int) -> None: ...
    def print_listen(self, format_str: str) -> None: ...
    def close(self) -> None: ...

class TcpWSGIServer(BaseWSGIServer):
    def bind_server_socket(self) -> None: ...
    def getsockname(self) -> tuple[str, str]: ...
    def set_socket_options(self, conn: socket) -> None: ...

if sys.platform != "win32":
    class UnixWSGIServer(BaseWSGIServer):
        def __init__(
            self,
            application: WSGIApplication,
            map: _SocketMap | None = None,
            _start: bool = True,
            _sock: socket | None = None,
            dispatcher: ThreadedTaskDispatcher | None = None,
            adj: Adjustments | None = None,
            sockinfo: tuple[int, int, int | None, _RetAddress] | None = None,
            **kw: _AdjustmentsParams,
        ) -> None: ...
        def bind_server_socket(self) -> None: ...
        def getsockname(self) -> tuple[str, str]: ...
        def fix_addr(self, addr: _RetAddress) -> tuple[Literal["localhost"], None]: ...

WSGIServer = TcpWSGIServer




from setuptools import setup

setup(package_data={'waitress-stubs': ['__init__.pyi', 'adjustments.pyi', 'buffers.pyi', 'channel.pyi', 'compat.pyi', 'parser.pyi', 'proxy_headers.pyi', 'receiver.pyi', 'rfc7230.pyi', 'runner.pyi', 'server.pyi', 'task.pyi', 'trigger.pyi', 'utilities.pyi', 'wasyncore.pyi', 'METADATA.toml', 'py.typed']})



from collections.abc import Sequence
from socket import _RetAddress, socket
from threading import Condition, Lock

from waitress.adjustments import Adjustments
from waitress.buffers import OverflowableBuffer
from waitress.parser import HTTPRequestParser
from waitress.server import BaseWSGIServer
from waitress.task import ErrorTask, WSGITask

from . import wasyncore
from .wasyncore import _SocketMap

class ClientDisconnected(Exception): ...

class HTTPChannel(wasyncore.dispatcher):
    task_class: type[WSGITask]
    error_task_class: type[ErrorTask]
    parser_class: type[HTTPRequestParser]
    request: HTTPRequestParser | None
    last_activity: float
    will_close: bool
    close_when_flushed: bool
    requests: Sequence[HTTPRequestParser]
    sent_continue: bool
    total_outbufs_len: int
    current_outbuf_count: int
    server: BaseWSGIServer
    adj: Adjustments
    outbufs: Sequence[OverflowableBuffer]
    creation_time: float
    sendbuf_len: int
    task_lock: Lock
    outbuf_lock: Condition
    connected: bool
    addr: _RetAddress
    def __init__(
        self, server: BaseWSGIServer, sock: socket, addr: _RetAddress, adj: Adjustments, map: _SocketMap | None = None
    ) -> None: ...
    def check_client_disconnected(self) -> None: ...
    def writable(self) -> bool: ...
    def handle_write(self) -> None: ...
    def readable(self) -> bool: ...
    def handle_read(self) -> None: ...
    def send_continue(self) -> None: ...
    def received(self, data: bytes) -> bool: ...
    def handle_close(self) -> None: ...
    def add_channel(self, map: _SocketMap | None = None) -> None: ...
    def del_channel(self, map: _SocketMap | None = None) -> None: ...
    def write_soon(self, data: bytes) -> int: ...
    def service(self) -> None: ...
    def cancel(self) -> None: ...



from re import Pattern
from typing import Final

BWS: Final = r"[ \t]{0,}?"
CHUNK_EXT: Final[str]
CHUNK_EXT_NAME: Final = r"[!#$%&'*+\-.^_`|~0-9A-Za-z]{1,}"
CHUNK_EXT_RE: Final[Pattern[str]]
CHUNK_EXT_VAL: Final[str]
DIGIT: Final = "[0-9]"
FIELD_CONTENT: Final[str]
FIELD_VALUE: Final[str]
FIELD_VCHAR: Final = r"[\x21-\x7e\x80-\xff]"
HEADER_FIELD_RE: Final[Pattern[str]]
HEXDIG: Final = r"[0-9a-fA-F]"
OBS_TEXT: Final = r"\x80-\xff"
ONLY_DIGIT_RE: Final[Pattern[str]]
ONLY_HEXDIG_RE: Final[Pattern[str]]
OWS: Final = r"[ \t]{0,}?"
QDTEXT: Final = r"[\t !#-[\]-~\x80-\xff]"
QUOTED_PAIR: Final = r"\\([\t \x21-\x7e\x80-\xff])"
QUOTED_PAIR_RE: Final[Pattern[str]]
QUOTED_STRING: Final[str]
QUOTED_STRING_RE: Final[Pattern[str]]
RWS: Final = r"[ \t]{1,}?"
TCHAR: Final = r"[!#$%&'*+\-.^_`|~0-9A-Za-z]"
TOKEN: Final = r"[!#$%&'*+\-.^_`|~0-9A-Za-z]{1,}"
VCHAR: Final = r"\x21-\x7e"
WS: Final = r"[ \t]"
